declare type SyncComputationInternal<T> = {
    type: 'Stream';
    tag: 'SyncComputation';
    value?: T;
    next?: T;
    compute: () => void;
};
declare type GeneratorComputationInternal<T> = {
    type: 'Stream';
    tag: 'GeneratorComputation';
    value?: T;
    next?: T;
    compute: () => void;
};
declare type ComputationInternal<T> = SyncComputationInternal<T> | GeneratorComputationInternal<T>;
declare type State = 'frozen' | 'propagating' | 'idle';
export declare const getPropagationState: () => State;
declare const eventListeners: {
    freezeEnd: Set<() => void>;
    propagatingEnd: Set<() => void>;
};
declare type InternalEvent = keyof typeof eventListeners;
export declare const removeEventListener: (event: InternalEvent, callback: () => void) => void;
export declare const addEventListener: (event: InternalEvent, callback: () => void) => void;
export declare const runOncePerTick: (callback: () => void) => void;
export declare class StreamError extends Error {
}
export declare class CleanupWithoutComputationContext extends StreamError {
}
export declare class Conflict extends StreamError {
}
export declare class ComputationWithoutRoot extends StreamError {
}
export declare class RunawayTicks extends StreamError {
}
export declare const MAX_TICKS = 100000;
export declare let runawayTicks: number;
/**
 * For tests and debugging.
 */
export declare const stats: {
    ticks: number;
    computations: {
        evaluated: number;
    };
};
/**
 * What the outside world sees/uses
 */
export interface Signal<T, U = T> {
    (): U;
    (x: T): U;
    (fn: ((x: U) => U)): U;
}
export declare type AnyMap<K, V> = K extends object ? WeakMap<K, V> | Map<K, V> : Map<K, V>;
export declare function xet<K, V>(map: AnyMap<K, V>, key: K, fn: (key: K, map: AnyMap<K, V>) => V): V;
export declare function on<T, U>(signal: Signal<any> | Signal<any>[], fn: (p: T) => U, seed: T, onchanges?: boolean): Computation<U>;
export declare function value<T>(value: T, equality?: EqualityCheck<T>): any;
declare type EqualityCheck<T> = (a: T, b: T) => boolean;
export declare function data<T>(value: T, equality: EqualityCheck<T>): Signal<T>;
export declare function data<T>(value: T): Signal<T>;
export declare function data<T>(value?: T): Signal<T | undefined>;
declare type SyncComputationVisitor<T> = (() => undefined) | (() => T) | ((previous: T) => T);
export declare type Computation<T> = () => T;
export declare const SKIP: {};
export declare function computation<T>(fn: SyncComputationVisitor<T>, seed: T): Computation<T>;
export declare function computation<T>(fn: SyncComputationVisitor<T>, seed?: T): Computation<T | undefined>;
interface StreamGeneratorVisitor<T> {
    (previous: T): any;
}
export declare function generator<T>(fn: StreamGeneratorVisitor<T>): Computation<T | undefined>;
export declare function generator<T>(fn: StreamGeneratorVisitor<T>, seed: T): Computation<T>;
export declare function freeze(f: VoidFunction): void;
export declare function cleanup(f: VoidFunction): void;
export declare function root<T>(f: (dispose: VoidFunction) => T): T;
export declare function parentScheduled(computation: ComputationInternal<unknown>, scheduledToRun: Set<ComputationInternal<unknown>>): boolean;
export declare function tick(): void;
export declare function sample<T>(signal: Signal<T>): T;
export declare function id(s: Signal<any>): string | undefined;
export {};
