import { data, runOncePerTick, sample, xet } from './index.js';
export { root } from './index.js';
import { dropRepeatsWith, map } from './utils.js';

const notify = new WeakMap();
const instances = new Map();
function prop(key) {
    const newPath = this.path.concat(String(key));
    return xet(instances, newPath.join('.'), () => createChildStore(row => [row[key]], (parent, update) => ({ ...parent, [key]: update(parent[key]) }), this, newPath));
}
function unnest() {
    const newPath = this.path.concat('unnest()');
    return xet(instances, newPath.join('.'), () => createChildStore(arrayRow => arrayRow, (arrayRow, update) => arrayRow.map(unnestedRow => update(unnestedRow)), this, newPath));
}
function filter(f) {
    const newPath = this.path.concat('filter(' + f + ')');
    return xet(instances, newPath.join('.'), () => createChildStore(row => f(row) ? [row] : [], (row, update) => (f(row) ? update(row) : row), this, newPath));
}
function focus(getter, setter) {
    const key = `focus(${getter})`;
    const newPath = this.path.concat(key);
    return xet(instances, newPath.join('.'), () => createChildStore(getter, setter, this, newPath));
}
function createStore(name, table) {
    // we use () => as equality because the result set changes all the
    // time, its not a legitimate way to detect conflicts, you'd have
    // to do some really deep comparisons to identify duplicate writes
    // and for this system it wouldn't mean anything
    //
    // leaf conflicts should probably be captured/detected in this layer
    // not in S, but any 2 writes will be techinically deemed a conflict 
    // because the state tree is being set to two different immutable
    // objects and result sets
    const stateStream = data(table, () => true);
    let lastWrite = table;
    let runOncePerTick$1 = () => {
        stateStream(lastWrite);
    };
    const write = (f) => {
        lastWrite = lastWrite.map(row => f(row));
        runOncePerTick(runOncePerTick$1);
    };
    const path = [name];
    let store = {
        sample: () => sample(stateStream)[0],
        sampleAll: () => sample(stateStream),
        write,
        read: () => stateStream()[0],
        readAll: () => stateStream(),
        getReadStream: () => stateStream,
        focus,
        prop,
        unnest,
        filter,
        path
    };
    notify.set(store, (f) => {
        lastWrite = lastWrite.map(row => f(row));
        runOncePerTick(runOncePerTick$1);
    });
    return store;
}
const compareResultSetReferences = (a, b) => {
    for (let i = 0; i < Math.max(a.length, b.length); i++) {
        if (a[i] !== b[i]) {
            return false;
        }
    }
    return true;
};
function createChildStore(getter, setter, parentStore, path) {
    const read$ = dropRepeatsWith(map((results) => {
        let out = results.flatMap(row => getter(row));
        return out;
    }, parentStore.getReadStream()), compareResultSetReferences);
    const write = (f) => {
        notify.get(parentStore)((parent) => setter(parent, f));
    };
    let store = {
        sample: () => sample(read$)[0],
        sampleAll: () => sample(read$),
        read: () => read$()[0],
        readAll: () => read$(),
        write,
        getReadStream: () => read$,
        focus,
        prop,
        unnest,
        filter,
        path
    };
    notify.set(store, (f) => {
        notify.get(parentStore)((parent) => setter(parent, f));
    });
    return store;
}

export { createStore };
//# sourceMappingURL=store.js.map
